
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '36.0.0rc3',
    'branch_nick': 'master',
    'revision_id': '2c8233b1a8185be5eb1575798c662c038e529ef0',
    'revno': 13327
}
